/***************************************************************
/  USB ADVANCE LOADER FOR PS3                                  /
/  Copyright (C) 2009 - Ifcaro <http://ps2dev.ifcaro.net>      /
/                                                              /
/  last mod: 26-04-2009                                        /
/  license:  GNU Library General Public License Version 2      /
/            (see LICENSE on the main directory)               /
***************************************************************/

#include <tamtypes.h>
#include <kernel.h>
#include <sifrpc.h>
#include <loadfile.h>
#include <stdio.h>

#include "libpad.h"

    int ret;
    int port, slot;
    int i;
    struct padButtonStatus buttons;
    u32 paddata;
    u32 old_pad = 0;
    u32 new_pad;

/*
 * Global var's
 */
// pad_dma_buf is provided by the user, one buf for each pad
// contains the pad's current state
static char padBuf[256] __attribute__((aligned(64)));

static char actAlign[6];
static int actuators;

/*
 * waitPadReady()
 */
int waitPadReady(int port, int slot)
{
    int state;
    int lastState;
    char stateString[16];

    state = padGetState(port, slot);
    lastState = -1;
    while((state != PAD_STATE_STABLE) && (state != PAD_STATE_FINDCTP1)) {
        if (state != lastState) {
            padStateInt2String(state, stateString);
            printf("Please wait, pad(%d,%d) is in state %s\n", 
                       port, slot, stateString);
        }
        lastState = state;
        state=padGetState(port, slot);
    }
    // Were the pad ever 'out of sync'?
    if (lastState != -1) {
        printf("Pad OK!\n");
    }
    return 0;
}


/*
 * initializePad()
 */
int
initializePad(int port, int slot)
{

    int ret;
    int modes;
    int i;

    waitPadReady(port, slot);

    // How many different modes can this device operate in?
    // i.e. get # entrys in the modetable
    modes = padInfoMode(port, slot, PAD_MODETABLE, -1);
    printf("The device has %d modes\n", modes);

    if (modes > 0) {
        printf("( ");
        for (i = 0; i < modes; i++) {
            printf("%d ", padInfoMode(port, slot, PAD_MODETABLE, i));
        }
        printf(")");
    }

    printf("It is currently using mode %d\n", 
               padInfoMode(port, slot, PAD_MODECURID, 0));

    // If modes == 0, this is not a Dual shock controller 
    // (it has no actuator engines)
    if (modes == 0) {
        printf("This is a digital controller?\n");
        return 1;
    }

    // Verify that the controller has a DUAL SHOCK mode
    i = 0;
    do {
        if (padInfoMode(port, slot, PAD_MODETABLE, i) == PAD_TYPE_DUALSHOCK)
            break;
        i++;
    } while (i < modes);
    if (i >= modes) {
        printf("This is no Dual Shock controller\n");
        return 1;
    }

    // If ExId != 0x0 => This controller has actuator engines
    // This check should always pass if the Dual Shock test above passed
    ret = padInfoMode(port, slot, PAD_MODECUREXID, 0);
    if (ret == 0) {
        printf("This is no Dual Shock controller??\n");
        return 1;
    }

    printf("Enabling dual shock functions\n");

    // When using MMODE_LOCK, user cant change mode with Select button
    padSetMainMode(port, slot, PAD_MMODE_DUALSHOCK, PAD_MMODE_LOCK);

    waitPadReady(port, slot);
    printf("infoPressMode: %d\n", padInfoPressMode(port, slot));

    waitPadReady(port, slot);        
    printf("enterPressMode: %d\n", padEnterPressMode(port, slot));

    waitPadReady(port, slot);
    actuators = padInfoAct(port, slot, -1, 0);
    printf("# of actuators: %d\n",actuators);

    if (actuators != 0) {
        actAlign[0] = 0;   // Enable small engine
        actAlign[1] = 1;   // Enable big engine
        actAlign[2] = 0xff;
        actAlign[3] = 0xff;
        actAlign[4] = 0xff;
        actAlign[5] = 0xff;

        waitPadReady(port, slot);
        printf("padSetActAlign: %d\n", 
                   padSetActAlign(port, slot, actAlign));
    }
    else {
        printf("Did not find any actuators.\n");
    }

    waitPadReady(port, slot);

    return 1;
}

int waitkey(){
	int rtn=0;
            
        ret = padRead(port, slot, &buttons); // port, slot, buttons
            
        if (ret != 0) {
            paddata = 0xffff ^ buttons.btns;
                
            new_pad = paddata & ~old_pad;
            old_pad = paddata;
                
            // Directions
            if(new_pad & PAD_LEFT) {
				rtn=1;
                //printf("LEFT\n");
            }
            if(new_pad & PAD_DOWN) {
				rtn=1;
                //printf("DOWN\n");
            }
            if(new_pad & PAD_RIGHT) {
				rtn=1;
                //printf("RIGHT\n");
            }
            if(new_pad & PAD_UP) {
				rtn=1;
                //printf("UP\n");
            }
            if(new_pad & PAD_START) {
				rtn=1;
                //printf("START\n");
            }
            if(new_pad & PAD_R3) {
				rtn=1;
                //printf("R3\n");
            }
            if(new_pad & PAD_L3) {
				rtn=1;
                //printf("L3\n");
            }
            if(new_pad & PAD_SELECT) {
				rtn=1;
                //printf("SELECT\n");
            }
            if(new_pad & PAD_SQUARE) {
				rtn=1;
                //printf("SQUARE\n");
            }
            if(new_pad & PAD_CROSS) {
				rtn=1;
                //printf("CROSS - Enter press mode\n");
            }
            if(new_pad & PAD_CIRCLE) {
				rtn=1;
                //printf("CIRCLE - Exit press mode\n");
            }
            if(new_pad & PAD_TRIANGLE) {
				rtn=1;
                //printf("TRIANGLE (press mode disabled, see code)\n");
            }
            if(new_pad & PAD_R1) {
				rtn=1;
                //printf("R1 - Start little engine\n");
            }
            if(new_pad & PAD_L1) { 
				rtn=1;            
                //printf("L1 - Stop little engine\n");
            }
            if(new_pad & PAD_R2) {
				rtn=1;
                //printf("R2\n");
            }
            if(new_pad & PAD_L2) {
				rtn=1;
                //printf("L2\n");
            }
        } 
	return rtn;
}

void updatepad(){
        ret = padRead(port, slot, &buttons); // port, slot, buttons
            
        if (ret != 0) {
            paddata = 0xffff ^ buttons.btns;
                
            new_pad = paddata & ~old_pad;
            old_pad = paddata;
		}
}

int getkey(int num){
	int rtn=0;
            
        /*ret = padRead(port, slot, &buttons); // port, slot, buttons
            
        if (ret != 0) {
            paddata = 0xffff ^ buttons.btns;
                
            new_pad = paddata & ~old_pad;
            old_pad = paddata;*/
                
            // Directions
            if(new_pad & PAD_LEFT) {
				if(num==1)rtn=1;
                //printf("LEFT\n");
            }
            if(new_pad & PAD_DOWN) {
				if(num==2)rtn=1;
                //printf("DOWN\n");
            }
            if(new_pad & PAD_RIGHT) {
				if(num==3)rtn=1;
                //printf("RIGHT\n");
            }
            if(new_pad & PAD_UP) {
				if(num==4)rtn=1;
                //printf("UP\n");
            }
            if(new_pad & PAD_START) {
				if(num==5)rtn=1;
                //printf("START\n");
            }
            if(new_pad & PAD_R3) {
				if(num==6)rtn=1;
                //printf("R3\n");
            }
            if(new_pad & PAD_L3) {
				if(num==7)rtn=1;
                //printf("L3\n");
            }
            if(new_pad & PAD_SELECT) {
				if(num==8)rtn=1;
                //printf("SELECT\n");
            }
            if(new_pad & PAD_SQUARE) {
				if(num==9)rtn=1;
                //printf("SQUARE\n");
            }
            if(new_pad & PAD_CROSS) {
				if(num==10)rtn=1;
                //printf("CROSS - Enter press mode\n");
            }
            if(new_pad & PAD_CIRCLE) {
				if(num==11)rtn=1;
                //printf("CIRCLE - Exit press mode\n");
            }
            if(new_pad & PAD_TRIANGLE) {
				if(num==12)rtn=1;
                //printf("TRIANGLE (press mode disabled, see code)\n");
            }
            if(new_pad & PAD_R1) {
				if(num==13)rtn=1;
                //printf("R1 - Start little engine\n");
            }
            if(new_pad & PAD_L1) { 
				if(num==14)rtn=1;            
                //printf("L1 - Stop little engine\n");
            }
            if(new_pad & PAD_R2) {
				if(num==15)rtn=1;
                //printf("R2\n");
            }
            if(new_pad & PAD_L2) {
				if(num==16)rtn=1;
                //printf("L2\n");
            }
        //} 
	return rtn;
}

void unloadpad(){
	padPortClose(port, slot);
	padReset();
}

int startpad()
{

    padInit(0);

    port = 0; // 0 -> Connector 1, 1 -> Connector 2
    slot = 0; // Always zero if not using multitap

	padGetPortMax();
	padGetSlotMax(port);
	
    //printf("PortMax: %d\n", padGetPortMax());
    //printf("SlotMax: %d\n", padGetSlotMax(port));
    
    if((ret = padPortOpen(port, slot, padBuf)) == 0) {
        printf("padOpenPort failed: %d\n", ret);
        SleepThread();
    }
    
    if(!initializePad(port, slot)) {
        printf("Error al iniciar el pad\n");
        SleepThread();
    }
	        ret=padGetState(port, slot);
        while((ret != PAD_STATE_STABLE) && (ret != PAD_STATE_FINDCTP1)) {
            if(ret==PAD_STATE_DISCONN) {
                printf("Pad(%d, %d) is disconnected\n", port, slot);
            }
            ret=padGetState(port, slot);
        }
        if(i==1) {
            //printf("Pad: OK!\n");
        }
    
    return 0;
}

